@@ -66,7 +66,7 @@ class Checksum:
                 data = open( file, 'rb' )
                 if HashType == "sha256":
                         Hash = self.sha256( data )
-                elif HashType == "md5":
+		elif HashType == "md5" or HashType == "md5sum":
                         Hash = self.md5( data )
                 else: Hash = None
                 data.close()
@@ -81,7 +81,7 @@ class Checksum:
                 return hash.hexdigest()
         
         def md5( self, data ):
-                hash = hashlib.md5.new()
+                hash = hashlib.md5()
                 hash.update( data.read() )
                 return hash.hexdigest() 
         
@@ -532,6 +532,8 @@ class MyThread( threading.Thread ):
                 self.requestQueue = requestQueue
                 self.responseQueue = responseQueue
                 self.threads = NumOfThreads
+                self.threads_finished = 0   # used by gui to understand if things got over
+                self.guiTerminateSignal=False
                 self.WorkerFunction = WorkerFunction
                 self.thread_pool = [
                        threading.Thread( 
@@ -540,6 +542,8 @@ class MyThread( threading.Thread ):
                               )
                        for i in range( self.threads )
                        ]
+                for thread in self.thread_pool:
+                       thread.guiTerminateSignal=False
         
         def startThreads( self ):
                 for thread in self.thread_pool:
@@ -554,18 +558,29 @@ class MyThread( threading.Thread ):
         def populateQueue( self, item ):
                 self.requestQueue.put( item )
                 
-        def stopQueue( self ):
+        def stopQueue( self, timeout=0 ):
                 '''Don't end the program prematurely.
                 (Note that because Queue.get() is blocking by
                 defualt this isn't strictly necessary. But if
                 you were, say, handling responses in another
                 thread, you'd want something like this in your
                 main thread.)'''
+                if timeout !=0:
+                        self.threads_finished = 0   # recount finished threads if gui handler needs
                 for thread in self.thread_pool:
-                        thread.join()
+                        if timeout==0: 
+                                thread.join()
+                        else:
+                                # let threads also lookout for gui signals of cancellation
+                                thread.join(timeout)
+                                if not thread.isAlive():
+                                        self.threads_finished += 1
                         
         def run( self, item=None):
                 while True:
+                        if threading.currentThread().guiTerminateSignal:
+                                #print threading.currentThread().getName(), "has been stopped :D"
+                                break
                         if self.requestQueue is not None:
                                 item = self.requestQueue.get()
                                 
@@ -579,4 +594,4 @@ class MyThread( threading.Thread ):
                                 exit_status = self.responseQueue.get()
                         else:
                                 self.WorkerFunction( item, thread_name )
-                                
\ No newline at end of file
+
